# -*- coding: utf-8 -*-

"""
Provides views for the gallery resources of pyGallerid.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp


import datetime
import json
import urllib
import itertools
import logging

from pyramid.location import lineage
from pyramid.view import view_config
from pyramid.httpexceptions import (
    HTTPFound,
    HTTPNotFound,
    HTTPForbidden,
)
from pyramid.traversal import find_resource, find_root, find_interface, \
     resource_path
from pyramid.security import authenticated_userid, remember, forget

from persistent.dict import PersistentDict
from persistent.list import PersistentList

from ..models import retrieve_about, retrieve_gallery, retrieve_user
from ..models.user import User
from ..models.gallery import GalleryContainer, \
     GalleryAlbum, GalleryPicture, GalleryDocument

logger = logging.getLogger(__name__)


#from pyramid.events import subscriber, NewRequest
#@subscriber(NewRequest)
#def newRequestSubscriber(event):
    #request = event.request
    #pass


def picture_item(method):
    def new_method(item):
        if isinstance(item, GalleryContainer):
            item = item.preview_picture
        #if item is None:
            #return ""
        return method(item)
    return new_method


def picture_request_item(method):
    def new_method(request, item):
        if isinstance(item, GalleryContainer):
            item = item.preview_picture
        #if item is None:
            #return ""
        return method(request, item)
    return new_method


@picture_request_item
def big_url(request, item):
    url = request.static_url(item.big_image_path)
    url = urllib.unquote(url)
    return url


@picture_item
def big_width(item):
    return item.big_image_view.width


@picture_item
def big_height(item):
    return item.big_image_view.height


@picture_request_item
def regular_url(request, item):
    url = request.static_url(item.regular_image_path)
    url = urllib.unquote(url)
    return url


@picture_item
def regular_width(item):
    return item.regular_image_view.width


@picture_item
def regular_height(item):
    return item.regular_image_view.height


@picture_request_item
def small_url(request, item):
    url = request.static_url(item.small_image_path)
    url = urllib.unquote(url)
    return url


@picture_item
def small_width(item):
    return item.small_image_view.width


@picture_item
def small_height(item):
    return item.small_image_view.height


def render_resource(resource):
    name = resource.name
    name = name[0].upper() + name[1:]
    return unicode(name)


def find_gallery_resource(resource):
    user = itertools.ifilter(
        lambda r: isinstance(r, User), lineage(resource)
    ).next()
    return retrieve_gallery(user)


def find_about_resource(resource):
    user = itertools.ifilter(
        lambda r: isinstance(r, User), lineage(resource)
    ).next()
    return retrieve_about(user)


@view_config(context=GalleryContainer, xhr=True, name='update',
             renderer='json', permission='update',
             request_param='pg-type=order-list')
def update_gallery_order_list(context, request):
    # TODO: implement CSRF
    #token = request.session.get_csrf_token()
    #if request.params['pg-csrf'] != token:
        #raise ValueError('CSRF token did not match!')
    #print 'JSON request with id=%s, name=%s' \
          #% (request.params['pg-id'], request.params['pg-name'])
    result = {'pg-status': 'failed'}
    pg_context = find_resource(context, request.params['pg-context'])
    pg_ids = json.loads(request.params['pg-value'])
    children = pg_context.children
    new_children = []
    for pg_id in pg_ids:
        new_children.append(children[pg_id])
    pg_context.children = new_children
    result['pg-status'] = 'success'
    result['pg-redirect-url'] = request.resource_url(context, '@@edit')
    #result['pg-redirect-url'] = request.resource_url(context)
    #result['pg-replace-url'] = request.resource_url(context)
    return result


@view_config(context=GalleryContainer, xhr=True, name='update',
             renderer='json', permission='update',
             request_param='pg-type=select-picture')
def update_gallery_select_picture(context, request):
    #print 'JSON request with id=%s, name=%s' \
          #% (request.params['pg-id'], request.params['pg-name'])
    result = {'pg-status': 'failed'}
    pg_context = find_resource(context, request.params['pg-context'])
    pg_resource = find_resource(
        pg_context,
        json.loads(request.params['pg-value'])
    )
    print 'selected picture %s' % pg_resource.name
    pg_context.preview_picture = pg_resource
    result['pg-status'] = 'success'
    result['pg-redirect-url'] = request.resource_url(context, '@@edit')
    #result['pg-redirect-url'] = request.resource_url(context)
    #result['pg-replace-url'] = request.resource_url(context)
    return result


@view_config(context=GalleryDocument, xhr=True, name='update',
             renderer='json', permission='update',
             request_param='pg-type=attribute-multiline-text')
@view_config(context=GalleryContainer, xhr=True, name='update',
             renderer='json', permission='update',
             request_param='pg-type=attribute-multiline-text')
@view_config(context=GalleryDocument, xhr=True, name='update',
             renderer='json', permission='update',
             request_param='pg-type=attribute-text')
@view_config(context=GalleryContainer, xhr=True, name='update',
             renderer='json', permission='update',
             request_param='pg-type=attribute-text')
def update_gallery_attribute_text(context, request):
    #print 'JSON request with id=%s, name=%s' \
          #% (request.params['pg-id'], request.params['pg-name'])
    result = {'pg-status': 'failed'}
    pg_context = find_resource(context, request.params['pg-context'])
    pg_name = request.params['pg-name']
    pg_text = json.loads(request.params['pg-value'])
    if find_gallery_resource(pg_context) == pg_context and pg_name == 'name':
        logger.info('Attempt to change name of gallery resource: %s' \
                    % pg_context)
        return result
    if find_about_resource(pg_context) == pg_context and pg_name == 'name':
        logger.info('Attempt to change name of about resource: %s' \
                    % pg_context)
        return result
    pg_context.__setattr__(pg_name, pg_text)
    if pg_context == context and pg_name == 'name':
        result['pg-redirect-url'] = request.resource_url(context, '@@edit')
        #result['pg-replace-url'] = request.resource_url(context)
    result['pg-status'] = 'success'
    return result


@view_config(context=GalleryContainer, xhr=True, name='update',
             renderer='json', permission='update',
             request_param='pg-type=attribute-date')
def update_gallery_attribute_date(context, request):
    #print 'JSON request with id=%s, name=%s' \
          #% (request.params['pg-id'], request.params['pg-name'])
    result = {'pg-status': 'failed'}
    pg_context = find_resource(context, request.params['pg-context'])
    pg_name = request.params['pg-name']
    pg_value = json.loads(request.params['pg-value'])
    date = datetime.datetime.strptime(pg_value, '%Y-%m-%d').date()
    pg_context.__setattr__(pg_name, date)
    result['pg-date'] = '%d %s %d' % (date.day, date.strftime('%B'), date.year)
    result['pg-status'] = 'success'
    return result


@view_config(context=GalleryContainer, xhr=True, name='update',
             renderer='json', permission='update',
             request_param='pg-type=attribute-date-from-to')
def update_gallery_attribute_date_from_to(context, request):
    #print 'JSON request with id=%s, name=%s' \
          #% (request.params['pg-id'], request.params['pg-name'])
    result = {'pg-status': 'failed'}
    pg_context = find_resource(context, request.params['pg-context'])
    pg_name = request.params['pg-name']
    pg_value = json.loads(request.params['pg-value'])
    date_from, date_to = [
        datetime.datetime.strptime(date, '%Y-%m-%d').date() \
        for date in (pg_value['from'], pg_value['to'])
    ]
    if date_to < date_from:
        raise AssertionError('date_to < date_from')
    if pg_name == '__date_from_to':
        pg_context.date_from = date_from
        pg_context.date_to = date_to
    else:
        pg_context.__setattr__(pg_name, (date_from, date_to))
    date_str = '%d' % date_from.day
    if date_from.month != date_to.month:
        date_str += ' %s' % date_from.strftime('%B')
    if date_from.year != date_to.year:
        date_str += ' %d' % date_from.year
    date_str += ' - %d %s %d' % (date_to.day, date_to.strftime('%B'),
                                 date_to.year)
    result['pg-date-from-to'] = date_str
    result['pg-status'] = 'success'
    return result

#@view_config(context=GalleryContainer, xhr=True,
             #name='update', permission='update',
             #renderer='json')
#def update_gallery(context, request):
    #result = {'pg-status': 'failed'}
    #print request.params
    #print 'JSON request with id=%s, name=%s' \
          #% (request.params['pg-id'], request.params['pg-name'])
    #pg_id = request.params['pg-id']
    #pg_name = request.params['pg-name']
    #pg_id = pg_id.split(':')
    #if pg_id[0] in ('Gallery', 'GalleryContainer', 'Album') \
       #and pg_id[1] == '':
        #if pg_name in ['date_from', 'date_to']:
            #date = datetime.datetime.strptime(
                #request.params['pg-value'], '%Y-%m-%d').date()
            #result['pg-date'] = '%d %s %d' \
                #% (date.day, date.strftime('%B'), date.year)
        #if pg_name == '__date_from_to':
            #date_from = datetime.datetime.strptime(
                #request.params['pg-value[from]'], '%Y-%m-%d').date()
            #date_to = datetime.datetime.strptime(
                #request.params['pg-value[to]'], '%Y-%m-%d').date()
            #if date_to < date_from:
                #raise AssertionError('date_to < date_from')
            #context.date_to = date_to
            #context.date_from = date_from
            #date_s = '%d' % date_from.day
            #if date_from.month != date_to.month:
                #date_s += ' %s' % date_from.strftime('%B')
            #if date_from.year != date_to.year:
                #date_s += ' %d' % date_from.year
            #date_s += ' - %d %s %d' % (date_to.day, date_to.strftime('%B'),
                                       #date_to.year)
            #result['pg-date-from-to'] = date_s
        #else:
            #context.__setattr__(pg_name, request.params['pg-value'])
        #result['pg-status'] = 'success'
        #return result
    #else:
        #if pg_id[1] in context:
            #context[pg_id[1]].__setattr__(pg_name, request.params['pg-value'])
            #result['pg-status'] = 'success'
            #return result
    #return result


@view_config(context=GalleryContainer, xhr=True, name='retrieve',
             renderer='json', request_param='pg-type=pictures',
             permission='view')
def retrieve_pictures(context, request):
    result = {'pg-status': 'failed'}
    pg_context = find_resource(context, request.params['pg-context'])
    if 'pg-slice' in request.params:
        pg_slice = [int(x) for x in request.params['pg-slice'].split(':')]
        pg_slice = slice(*pg_slice)
    else:
        pg_slice = slice(None)
    indices = range(len(pg_context))[pg_slice]
    pg_pictures = []
    for index, child in enumerate(pg_context.itervalues()):
        if index not in indices:
            continue
        if isinstance(child, GalleryContainer):
            thumbnail = child.preview_picture
        elif isinstance(child, GalleryPicture):
            thumbnail = child
        else:
            raise Exception("Can't retrieve thumbnails "
                            "for this kind of object")
        pg_pictures.append({
            'index': index,
            'name': child.name,
            'big_url': big_url(request, thumbnail),
            'regular_url': regular_url(request, thumbnail),
            'width': regular_width(thumbnail),
            'height': regular_height(thumbnail),
        })
    result['pg-pictures'] = json.dumps(pg_pictures)
    result['pg-status'] = 'success'
    return result


@view_config(context=GalleryContainer, xhr=True, name='retrieve',
             renderer='json', request_param='pg-type=thumbnails',
             permission='view')
def retrieve_thumbnails(context, request):
    result = {'pg-status': 'failed'}
    pg_context = find_resource(context, request.params['pg-context'])
    pg_thumbnails = []
    for index, child in enumerate(pg_context.itervalues()):
        if isinstance(child, GalleryContainer):
            thumbnail = child.preview_picture
        elif isinstance(child, GalleryPicture):
            thumbnail = child
        else:
            raise Exception("Can't retrieve thumbnails "
                            "for this kind of object")
        pg_thumbnails.append({
            'index': index,
            'name': child.name,
            'url': small_url(request, thumbnail),
            'width': small_width(thumbnail),
            'height': small_height(thumbnail),
        })
    result['pg-thumbnails'] = json.dumps(pg_thumbnails)
    result['pg-status'] = 'success'
    return result


def convert_resource_to_json(resource, state='closed'):
    json_resource = {}
    json_resource['state'] = state
    if resource == find_root(resource):
        json_resource['data'] = '<root>'
    else:
        json_resource['data'] = str(resource)
    json_resource['attr'] = {'pg-path': resource_path(resource)}
    metadata = {}
    if hasattr(resource, '__attributes__'):
        for attr_name in resource.__attributes__:
            metadata[attr_name] = str(getattr(resource, attr_name))
    json_resource['metadata'] = metadata
    return json_resource


@view_config(context=GalleryDocument, name='retrieve',
             renderer='json', request_param='pg-type=resource-lineage',
             permission='edit')
@view_config(context=GalleryContainer, name='retrieve',
             renderer='json', request_param='pg-type=resource-lineage',
             permission='edit')
def retrieve_resource_lineage(context, request):
    result = {'pg-status': 'failed'}

    def convert_lineage_to_json(resource, descendants):
        json_resource = convert_resource_to_json(
            resource, state='open')
        if isinstance(resource, PersistentDict):
            iterator = resource.itervalues()
        elif isinstance(resource, PersistentList):
            iterator = resource.__iter__()
        else:
            iterator = ().__iter__()
        children = []
        for child in iterator:
            if len(descendants) > 0 and child == descendants[0]:
                json_child = convert_lineage_to_json(
                    child, descendants[1:])
            else:
                json_child = convert_resource_to_json(
                    child, state='closed')
            children.append(json_child)
        json_resource['children'] = children
        return json_resource

    descendants = list(lineage(context))[::-1]
    root = descendants[0]
    pg_lineage = convert_lineage_to_json(root, descendants[1:])
    result['pg-data'] = json.dumps(pg_lineage)
    result['pg-status'] = 'success'
    return result


@view_config(context=GalleryDocument, name='retrieve',
             renderer='json', request_param='pg-type=resource-children',
             permission='edit')
@view_config(context=GalleryContainer, name='retrieve',
             renderer='json', request_param='pg-type=resource-children',
             permission='edit')
def retrieve_resource_children(context, request):
    result = {'pg-status': 'failed'}
    resource = find_resource(find_root(context), request.params['pg-path'])
    if isinstance(resource, PersistentDict):
        pg_children = [ \
            convert_resource_to_json(child) \
                for child in resource.itervalues()
        ]
    elif isinstance(resource, PersistentList):
        pg_children = [ \
            convert_resource_to_json(child) \
                for child in resource
        ]
    else:
        pg_children = []
    result['pg-data'] = json.dumps(pg_children)
    result['pg-status'] = 'success'
    return result


@view_config(context=GalleryContainer, name='login',
             permission='login')
@view_config(context=GalleryDocument, name='login',
             permission='login')
def login(context, request):
    username = request.params['username']
    password = request.params['password']
    root = find_root(context)
    user = retrieve_user(root, username)
    if user is not None and user.authenticate(password):
        headers = remember(request, username)
        request.session.flash('Logged in successfully')
    else:
        headers = None
        request.session.flash('Login failed')
    #resp = render_view_to_response(context)
    #return resp
    url = request.resource_url(context)
    return HTTPFound(location=url, headers=headers)
    #response = view_gallery(context, request)
    #response.headers.update(hreaders)
    #return response


@view_config(context=GalleryContainer, name='logout',
             permission='logout')
def logout(context, request):
    headers = forget(request)
    request.session.flash('Logged out')
    #resp = render_view_to_response(context)
    #return resp
    url = request.resource_url(context)
    return HTTPFound(location=url, headers=headers)


def allow_editing(context, request):
    user = find_interface(context, User)
    if user is not None \
       and authenticated_userid(request) == user.name:
        return True
    else:
        return False


def about_url(context, request):
    about_resource = find_about_resource(context)
    if about_resource != context:
        about_url = request.resource_url(about_resource,
                                         '@@' + request.view_name)
    else:
        about_url = None
    return about_url


@view_config(context=GalleryContainer, name='edit',
             renderer='view_gallery.html.mako', permission='edit')
def edit_gallery(context, request):
    d = view_gallery(context, request)
    d.update({'editing': True})
    return d


@view_config(context=GalleryContainer,
             renderer='view_gallery.html.mako', permission='view')
def view_gallery(context, request):
    # TODO: do something else about empty containers
    items = []
    for item in context.itervalues():
        if len(item) > 0:
            items.append(item)
        #else:
            #print 'empty album:', item.name
    items = list(enumerate(items))
    #items = list(enumerate(context))

    local_small_url = lambda item: small_url(request, item)

    return {
        'messages': request.session.pop_flash(),
        'user': request.user,
        'gallery': context,
        'items': items,
        'allow_editing': allow_editing(context, request),
        'editing': False,
        'lineage_list': list(lineage(context))[:-2],
        'about_url': about_url(context, request),
        'render_resource': render_resource,
        'small_url': local_small_url,
        'small_width': small_width,
        'small_height': small_height,
    }


@view_config(context=GalleryAlbum, name='edit',
             renderer='view_album.html.mako', permission='edit')
def edit_album(context, request):
    d = view_album(context, request)
    d.update({'editing': True})
    return d


@view_config(context=GalleryAlbum,
             renderer='view_album.html.mako', permission='view')
def view_album(context, request):
    display_mode = request.params.get('display_mode', 'list')
    if display_mode == 'list':
        pictures_per_page = 10
    elif display_mode == 'grid':
        pictures_per_page = 20
    else:
        pictures_per_page = -1
    if pictures_per_page > 0:
        num_of_pages = len(context) / pictures_per_page + 1
        try:
            page = int(request.params.get('page', 1))
        except ValueError:
            page = 1
        if page < 1 or page > num_of_pages:
            page = 1
        pictures_start_index = (page - 1) * pictures_per_page
        pictures_end_index = pictures_start_index + pictures_per_page
    else:
        page = 1
        num_of_pages = 1
        pictures_start_index = 0
        pictures_end_index = len(context.pictures)
    pictures = context.pictures[pictures_start_index:pictures_end_index]
    pictures = zip(
        xrange(pictures_start_index, pictures_end_index),
        pictures
    )

    local_big_url = lambda item: big_url(request, item)
    local_regular_url = lambda item: regular_url(request, item)
    local_small_url = lambda item: small_url(request, item)

    return {
        'messages': request.session.pop_flash(),
        'user': request.user,
        'album': context,
        'allow_editing': allow_editing(context, request),
        'editing': False,
        'lineage_list': list(lineage(context))[:-2],
        'about_url': about_url(context, request),
        'render_resource': render_resource,
        'pictures': pictures,
        'display_mode': display_mode,
        'page': page,
        'num_of_pages': num_of_pages,
        'total_num_of_pictures': len(context),
        'small_url': local_small_url,
        'small_width': small_width,
        'small_height': small_height,
        'regular_url': local_regular_url,
        'regular_width': regular_width,
        'regular_height': regular_height,
        'big_url': local_big_url,
    }


@view_config(context=GalleryDocument, name='edit',
             renderer='view_document.html.mako', permission='edit')
def edit_document(context, request):
    d = view_document(context, request)
    d.update({'editing': True})
    return d


@view_config(context=GalleryDocument,
             renderer='view_document.html.mako', permission='view')
def view_document(context, request):
    lineage_list = [context, find_gallery_resource(context)]
    return {
        'messages': request.session.pop_flash(),
        'user': request.user,
        'document': context,
        'allow_editing': allow_editing(context, request),
        'editing': False,
        'lineage_list': lineage_list,
        'about_url': about_url(context, request),
        'render_resource': render_resource,
    }



# -*- coding: utf-8 -*-

"""
Script for importing pictures into the pyGallerid database.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp


import os
import sys

# this is not documented in the pyramid 1.3 documentation,
# so if it disappears at some point just use repoze.zodbconn
# or connect to the ZODB directly
from pyramid_zodbconn import db_from_uri

#import repoze.zodbconn.uri
#from ZODB.FileStorage import FileStorage
#from ZODB.DB import DB

import transaction

from pyramid.paster import (
    get_appsettings,
    setup_logging,
)

from ..models import appmaker, retrieve_user, retrieve_gallery
from ..utils.picture import import_gallery_container


def usage(argv):
    cmd = os.path.basename(argv[0])
    print('usage: %s <config_uri> <path> <sorting_order>\n'
          '(example: "%s development.ini new_pictures [text|number|date]' \
          % (cmd, cmd))
    sys.exit(1)


def main(argv=sys.argv):
    if len(argv) != 3:
        usage(argv)
    config_uri = argv[1]
    path = argv[2]
    sorting_order = 'number'
    if len(argv) > 3:
        sorting_order = argv[3]
    setup_logging(config_uri)
    settings = get_appsettings(config_uri)
    # for SQLalchemy
    #engine = engine_from_config(settings, 'sqlalchemy.')
    #DBSession.configure(bind=engine)
    #Base.metadata.create_all(engine)
    db = db_from_uri(settings['zodbconn.uri'])
    # for ZODB without repoze.zodbconn
    #storage = FileStorage(settings['zodbconn.file'])
    #db = DB(storage)
    conn = db.open()
    zodb_root = conn.root()
    with transaction.manager:
        import_pictures(zodb_root, settings, path, sorting_order)
        transaction.commit()


def import_pictures(zodb_root, settings, path, sorting_order):
    username = settings['default_user']
    app = appmaker(zodb_root)
    user = retrieve_user(app, username)
    gallery = retrieve_gallery(user)

    cwd = os.getcwd()
    os.chdir(path)

    container = import_gallery_container('.', settings, move_files=False, sorting_order=sorting_order)

    os.chdir(cwd)

    if container is not None:
        for child in container:
            gallery.add(child)
        #albums = category.children
        #albums.sort(cmp=lambda x, y: cmp(x.date_from, y.date_from))
        #category.children = albums




# -*- coding: utf-8 -*-

"""
Provides models for the user resources of pyGallerid.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp


import random
import hashlib
import string

from pyramid.traversal import find_root
from pyramid.security import Everyone, Allow

from . import PersistentContainer

HASHCLASS = hashlib.sha512
HASHSIZE = 2 * HASHCLASS().digest_size
SALTSIZE = HASHSIZE


class User(PersistentContainer):
    __attributes__ = ['email', 'password_hash', 'password_salt']

    def __init__(self, name, email, password, parent=None):
        PersistentContainer.__init__(self, name, parent)
        self.email = email
        self.password_hash, self.password_salt = User.hash_password(password)
        self.__acl__ = [
            (Allow, Everyone, 'view'),
            (Allow, Everyone, 'login'),
            (Allow, Everyone, 'logout'),
            (Allow, name, 'edit'),
            (Allow, name, 'update'),
        ]

    def authenticate(self, password):
        self.__acl__ = [
            (Allow, Everyone, 'view'),
            (Allow, Everyone, 'login'),
            (Allow, Everyone, 'logout'),
            (Allow, self.name, 'edit'),
            (Allow, self.name, 'update'),
        ]
        hexhash = HASHCLASS(password + self.password_salt)
        return hexhash.hexdigest() == self.password_hash

    @staticmethod
    def hash_password(password):
        password_salt = ''.join(
            random.choice(string.printable) for x in xrange(SALTSIZE)
        )
        hexhash = HASHCLASS(password + password_salt)
        return hexhash.hexdigest(), password_salt


def groupfinder(username, request):
    root = find_root(request.context)
    if username in root:
        return ()
    else:
        #raise KeyError('Unknown user: %s' % username)
        return None
    #return username



# -*- coding: utf-8 -*-

"""
repoze.evolution package for the pyGallerid database.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp


from collections import deque

from persistent import Persistent
from persistent.dict import PersistentDict
from persistent.list import PersistentList

def walk_resources(root):
    dq = deque()
    dq.append(root)
    while len(dq) > 0:
        resource = dq.popleft()
        if isinstance(resource, Persistent):
            yield resource
        if isinstance(resource, PersistentDict):
            for name in resource:
                dq.append(resource[name])
        elif isinstance(resource, PersistentList):
            for child in resource:
                dq.append(child)



# -*- coding: utf-8 -*-

"""
Provides tests for the models of pyGallerid.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp


import unittest
import transaction

from pyramid import testing

from ..models import appmaker
from ..models.user import User
from ..models.gallery import Gallery, GalleryContainer, GalleryAlbum, GalleryPicture

class ViewTests(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()

    def tearDown(self):
        testing.tearDown()

    def test_my_view(self):
        from .views import my_view
        request = testing.DummyRequest()
        info = my_view(request)
        self.assertEqual(info['project'], 'gallery')

class ModelTests(unittest.TestCase):

    def setUp(self):
        self.config = testing.setUp()
        from sqlalchemy import create_engine
        from sqlalchemy.orm import sessionmaker
        engine = create_engine('sqlite:///:memory:')
        DBSession.configure(bind=engine)
        from zope.sqlalchemy import ZopeTransactionExtension
        Base.metadata.create_all(engine)
        with transaction.manager:
            from ..scripts.populate import test_fillDB
            test_fillDB(DBSession())

    def tearDown(self):
        DBSession.remove()
        testing.tearDown()

    def test_UserModel(self):
        pass
    
    def test_AlbumModel(self):
        pass

    def test_PictureModel(self):
        pass

def run():
    unittest.main()

if __name__ == '__main__':
    run()




# -*- coding: utf-8 -*-

"""
Provides views for the root resource and the favicon of pyGallerid.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp


import os

from pyramid.response import Response
from pyramid.view import view_config
from pyramid.httpexceptions import (
    HTTPFound,
    HTTPNotFound,
    HTTPForbidden,
)

from ..models import retrieve_user, retrieve_gallery, PersistentContainer


## Load favicon.ico
#_here = os.path.dirname(__file__)
#_icon = open(os.path.join(_here, '../static', 'favicon.ico')).read()
#_icon_response = Response(content_type='image/x-icon', body=_icon)


#@view_config(name='favicon.ico')
#def favicon_view(context, request):
    #return _icon_response


#@view_config(context=PersistentContainer)
#def root(context, request):
    #user = retrieve_user(
        #context, request.registry.settings.get('default_user', 'root')
    #)
    #gallery = retrieve_gallery(user)
    #if gallery is not None:
        #return HTTPFound(location=request.resource_url(gallery))
    #else:
        #return HTTPNotFound()



# -*- coding: utf-8 -*-

"""
Script for initializing the pyGallerid database.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp


import os
import sys

# this is not documented in the pyramid 1.3 documentation,
# so if it disappears at some point just use repoze.zodbconn
# or connect to the ZODB directly
from pyramid_zodbconn import db_from_uri

#import repoze.zodbconn.uri
#from ZODB.FileStorage import FileStorage
#from ZODB.DB import DB

import transaction

from pyramid.paster import (
    get_appsettings,
    setup_logging,
)

from ..models import appmaker, retrieve_user, retrieve_gallery, retrieve_about
from ..models.user import User
from ..models.gallery import Gallery, GalleryDocument


def usage(argv):
    cmd = os.path.basename(argv[0])
    print('usage: %s <config_uri> <email> '
          '<password> [<description>]\n'
          '(example: "%s development.ini myname '
          'mymail@mydomain.com qwerty)' % (cmd, cmd))
    sys.exit(1)


def main(argv=sys.argv):
    if len(argv) < 4:
        usage(argv)
    config_uri = argv[1]
    email = argv[2]
    password = argv[3]
    if len(argv) > 4:
        description = argv[4]
    else:
        description = "Benjamin Hepp's Photography"
    setup_logging(config_uri)
    settings = get_appsettings(config_uri)
    # for SQLalchemy
    #engine = engine_from_config(settings, 'sqlalchemy.')
    #DBSession.configure(bind=engine)
    #Base.metadata.create_all(engine)
    db = db_from_uri(settings['zodbconn.uri'])
    # for ZODB without repoze.zodbconn
    #storage = FileStorage(settings['zodbconn.file'])
    #db = DB(storage)
    conn = db.open()
    zodb_root = conn.root()
    with transaction.manager:
        create_gallery(zodb_root, settings, email,
                       password, description)
        transaction.commit()


def create_gallery(zodb_root, settings, email,
                   password, description):
    username = settings['default_user']
    app = appmaker(zodb_root)
    user = retrieve_user(app, username)
    if user is not None:
        raise ValueError("A user with name %s already exists" % username)
    #password_hash, password_salt = User.hash_password(password)
    #user = User(username, email, password_hash, password_salt)
    user = User(username, email, password)
    app.add(user)
    gallery = retrieve_gallery(user)
    if gallery is not None:
        raise ValueError("The user %s already has a gallery" % username)
    gallery = Gallery(description, user=user, path=settings['image_dir'])
    user.add(gallery)
    about = retrieve_about(user)
    if about is not None:
        raise ValueError("The user %s already has an about page" % username)
    about = GalleryDocument('about', 'About', 'Description')
    user.add(about)



# -*- coding: utf-8 -*-

"""
repoze.evolution script for the pyGallerid database.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp


from . import walk_resources

def evolve(context):
    for resource in walk_resources(context):
        cls = type(resource)
        if hasattr(cls, '__attributes__') \
           and not hasattr(resource, '__attributes__'):
            resource.__attributes__ = cls.__attributes__



# -*- coding: utf-8 -*-

"""
Provides helpers for handling and importing image files.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp


import os
import re
import dateutil.parser
import subprocess
import shutil
import Image
from ExifTags import TAGS

from ..models.gallery import GalleryContainer, GalleryAlbum, \
     GalleryPicture, GalleryImageFile

DEFAULT_MIN_THUMBNAIL_SIZE = 400
DEFAULT_DISPLAY_SCALE = 0.5

MAX_REGULAR_VIEW_WIDTH = 1024
MAX_REGULAR_VIEW_HEIGHT = 536

MAX_SMALL_VIEW_WIDTH = 300
MAX_SMALL_VIEW_HEIGHT = 300


def open_picture(filename, return_tags=True):
    img = Image.open(filename)
    if return_tags:
        #from ExifTags import TAGS
        info = img._getexif()
        tags = {}
        for tag, value in info.items():
            decoded = TAGS.get(tag, tag)
            tags[decoded] = value
    else:
        tags = None
    return img, tags


def get_scaled_size(width, height, max_width, max_height):
    scale = max_width / float(width)
    if round(height * scale) > max_height:
        scale = max_height / float(height)
    width = int(round(width * scale))
    height = int(round(height * scale))
    return (width, height)


def create_gallery_image_file(filename, tags=None):
    img, new_tags = open_picture(filename, return_tags=(tags is None))
    if tags is None:
        tags = new_tags
    width, height = img.size
    image = GalleryImageFile(
        filename,
        size=(width, height),
        tags=tags
    )
    return image


def create_regular_gallery_image_file(filename, output_filename,
                                      width, height, tags,
                                      use_image_magick=True):
    new_size = get_scaled_size(
        width, height,
        MAX_REGULAR_VIEW_WIDTH, MAX_REGULAR_VIEW_HEIGHT)
    if use_image_magick:
        try:
            subprocess.check_call(['convert', filename, '-resize',
                                   '%dx%d!' % new_size,
                                   output_filename])
        except subprocess.CalledProcessError as e:
            print 'ImageMagick failed with errorcode %d' % e.returncode
            print '\n'.join(['  %s' % line for line in '\n'.split(e.output)])
            raise
    else:
        img = Image.open(filename)
        img = img.resize(new_size, Image.ANTIALIAS)
        img.save(output_filename)
    return create_gallery_image_file(output_filename, tags)


def create_small_gallery_image_file(filename, output_filename,
                                    width, height, tags,
                                    use_image_magick=True):
    new_size = get_scaled_size(
        width, height,
        MAX_SMALL_VIEW_WIDTH, MAX_SMALL_VIEW_HEIGHT)
    if use_image_magick:
        try:
            subprocess.check_call(['convert', filename, '-resize',
                                   '%dx%d!' % new_size,
                                   output_filename])
        except subprocess.CalledProcessError as e:
            print 'ImageMagick failed with errorcode %d' % e.returncode
            print '\n'.join(['  %s' % line for line in '\n'.split(e.output)])
            raise
    else:
        img = Image.open(filename)
        img = img.resize(new_size, Image.ANTIALIAS)
        img.save(output_filename)
    return create_gallery_image_file(output_filename, tags)


def import_gallery_picture(original_filename, big_filename, regular_filename,
                           small_filename, default_date=None, move_file=True,
                           use_image_magick=True):

    if os.path.abspath(original_filename) != os.path.abspath(big_filename):
        if move_file:
            shutil.move(original_filename, big_filename)
        else:
            shutil.copy2(original_filename, big_filename)

    img, tags = open_picture(big_filename)

    big_image = create_gallery_image_file(big_filename, tags)
    width, height = big_image.width, big_image.height

    regular_image = create_regular_gallery_image_file(
        big_filename, regular_filename, width, height, use_image_magick)
    small_image = create_small_gallery_image_file(
        big_filename, small_filename, width, height, use_image_magick)

    dates = []
    for tag in ('DateTime', 'DateTimeOriginal', 'DateTimeDigitized'):
        if tag in tags:
            dates.append(dateutil.parser.parse(tags[tag]))
    if len(dates) > 0:
        date = min(dates)
    else:
        if default_date is None:
            raise ValueError('No date found for:', original_filename)
        else:
            logging.warning('No date found: %s' % original_filename)
            date = default_date

    name = os.path.splitext(os.path.basename(original_filename))[0]
    description = name
    location = None

    picture = GalleryPicture(name, big_image, regular_image, small_image,
                             description, location, date)

    return picture


def import_gallery_album(album_path, settings, move_files=True,
                         use_image_magick=True, sorting_order='number'):

    rel_album_path = os.path.basename(album_path)

    datematcher = re.compile(
        r'^([\d]{4}) ([a-zA-Z0-9]{1,9}) ([\d]{1,2})(?:\-([\d]{1,2}))? (.+)$')
    try:
        mo = datematcher.match(rel_album_path)
        year, month, day_from, day_to, album_name = mo.groups()
        date_from = dateutil.parser.parse(
            '%s %s %s' % (year, month, day_from)).date()
        date_to = dateutil.parser.parse(
            '%s %s %s' % (year, month, day_to)).date()
        print '  album date: from %s to %s' % (date_from, date_to)
    except (AttributeError, ValueError):
        logging.warning('Unable to extract dates: %s' % rel_album_path)
        raise

    big_image_dir = os.path.join(settings['image_dir'], album_path, 'big')
    regular_image_dir = os.path.join(
        settings['image_dir'], album_path, 'regular')
    small_image_dir = os.path.join(settings['image_dir'], album_path, 'small')
    for path in (big_image_dir, regular_image_dir, small_image_dir):
        if not os.path.exists(path):
            os.makedirs(path)

    print 'scanning %s' % album_path
    pictures = []
    for filename in os.listdir('.'):
        if filename.startswith('.'):
            continue
        filebase, fileext = os.path.splitext(filename)
        if os.path.isfile(filename) and fileext.lower() in \
            ['.jpg', '.jpeg', '.png', '.tif', '.tiff']:
            print '  importing %s' % filename
            big_filename = os.path.join(big_image_dir, filename)
            regular_filename = os.path.join(regular_image_dir, filename)
            small_filename = os.path.join(small_image_dir, filename)
            try:
                picture = import_gallery_picture(
                    filename, big_filename,
                    regular_filename, small_filename,
                    move_file=move_files)
            except:
                print 'Failed to import picture:', \
                      os.path.join(album_path, filename)
                raise
            pictures.append(picture)

    if len(pictures) == 0:
        return None

    album = GalleryAlbum(album_name, album_name, album_name,
                         None, date_from, date_to, rel_album_path)
    if sorting_order == 'date':
        pictures.sort(cmp=lambda x, y: cmp(x.date, y.date))
    elif sorting_order == 'number':
        pattern = re.compile('(.+?)([0-9]+).*?')

        def match_picture(picture):
            mo = pattern.match(picture.name)
            return mo.group(1), int(mo.group(2))

        def cmp_pictures(picture1, picture2):
            prefix1, num1 = match_picture(picture1)
            prefix2, num2 = match_picture(picture2)
            if cmp(prefix1, prefix2) != 0:
                return cmp(prefix1, prefix2)
            else:
                return cmp(num1, num2)
        pictures.sort(cmp=cmp_pictures)
    else:
        pictures.sort(cmp=lambda x, y: cmp(x.name, y.name))

    for picture in pictures:
        # make image file paths relative
        for image, image_dir in ( \
            (picture.big_image_view.image, big_image_dir),
            (picture.regular_image_view.image, regular_image_dir),
            (picture.small_image_view.image, small_image_dir),
        ):
            head, tail = os.path.split(image.filename)
            head2, tail2 = os.path.split(head)
            image.filename = os.path.join(tail2, tail)
        # add picture to album container
        album.append(picture)

    return album


def import_gallery_container(path, settings, move_files=True,
                             use_image_magick=True, sorting_order='number'):
    cwd = os.getcwd()
    os.chdir(os.path.split(path)[-1])
    album = None
    containers = []
    picture_found = False
    print 'scanning %s' % path
    for filename in os.listdir('.'):
        if filename.startswith('.'):
            continue
        container = None
        filebase, fileext = os.path.splitext(filename)
        if not picture_found and os.path.isfile(filename) \
           and fileext.lower() in \
           ['.jpg', '.jpeg', '.png', '.tif', '.tiff']:
            try:
                album = import_gallery_album(
                    path, settings, move_files, use_image_magick)
            except:
                print 'Failed to import album:', path
                raise
            picture_found = True
        elif os.path.isdir(filename):
            abs_path = os.path.join(path, filename)
            try:
                container = import_gallery_container(
                    abs_path, settings, move_files,
                    use_image_magick, sorting_order)
            except:
                print 'Failed to import container:', abs_path
                raise
            if container is not None:
                containers.append(container)
    os.chdir(cwd)
    if len(containers) > 0:
        if album is None:
            rel_path = os.path.basename(path)
            new_container = GalleryContainer(
                rel_path, rel_path, None, rel_path)
        else:
            new_container = album
        [new_container.add(c) for c in containers]
        return new_container
    elif album is not None:
        return album



# -*- coding: utf-8 -*-

"""
Provides common models and helpers for resources of pyGallerid.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp


from persistent.dict import PersistentDict
from persistent.list import PersistentList

__sw_version__ = 1


class PersistentLocationAware(object):
    __name__ = None
    __parent__ = None

    def __init__(self, name, parent=None):
        self.__name__ = name
        self.__parent__ = parent
        if parent is not None:
            parent[name] = self

    def __str__(self):
        return self.name

    @property
    def name(self):
        return self.__name__

    @name.setter
    def name(self, value):
        if self.parent is not None:
            p = self.parent
            self.parent = None
            #del parent[self.name]
            self.__name__ = value
            self.parent = p
            #p[value] = self
        else:
            raise AttributeError('The name of a root object can\'t be changed')

    @property
    def parent(self):
        return self.__parent__

    @parent.setter
    def parent(self, obj):
        if self.parent is not None:
            p = self.parent
            self.__parent__ = None
            del p[self.name]
        if obj is not None:
            obj[self.name] = self
            self.__parent__ = obj


class PersistentContainer(PersistentDict, PersistentLocationAware):
    def __init__(self, name, parent=None):
        PersistentDict.__init__(self)
        PersistentLocationAware.__init__(self, name, parent)
        self.__name__ = name
        if parent is not None:
            parent[name] = self
        self.__parent__ = parent

    def __add(self, item):
        PersistentDict.__setitem__(self, item.__name__, item)
        item.__parent__ = self
    add = __add

    def __setitem__(self, name, item):
        if item.__name__ != name:
            raise ValueError('name and item.__name__ must be equal')
        self.__add(item)

    def __delitem__(self, name):
        self[name].__parent__ = None
        PersistentDict.__delitem__(self, name)


class PersistentOrderedContainer(PersistentContainer):
    def __init__(self, name, parent=None):
        PersistentContainer.__init__(self, name, parent)
        self.__children = PersistentList()

    def index(self, item):
        return self.__children.index(item)

    def get_children(self, index):
        return self.__children[index]

    @property
    def children(self):
        return list(self.__children)

    @children.setter
    def children(self, children):
        if len(children) != len(self):
            raise ValueError('len(children) and len(self) must be equal')
        for child in children:
            if not child.name in self:
                raise ValueError('children and self must ' \
                                 'contain the same objects')
        self.__children = PersistentList(children)

    def add(self, item):
        if item.__name__ not in self:
            self.__children.append(item)
        else:
            raise ValueError('The container already contains this item')
        PersistentContainer.add(self, item)
    append = add

    def insert(self, index, item):
        if item.__name__ not in self:
            self.__children.insert(index, item)
        else:
            raise ValueError('The container already contains this item')
        PersistentContainer.append(self, item)

    def __setitem__(self, name, item):
        already_in_children = name in self
        PersistentContainer.__setitem__(self, name, item)
        if not already_in_children:
            self.__children.append(item)

    def __delitem__(self, name):
        if name in self:
            self.__children.remove(self[name])
        PersistentContainer.__delitem__(self, name)

    def __iter__(self):
        return self.iterkeys()

    def keys(self):
        return [child.name for child in self.__children]

    def values(self):
        return [child for child in self.__children]

    def items(self):
        return [(child.name, child) for child in self.__children]

    def iterkeys(self):
        for child in self.__children:
            yield child.name

    def itervalues(self):
        for child in self.__children:
            yield child

    def iteritems(self):
        for child in self.__children:
            yield child.name, child


def retrieve_user(app, username):
    if username in app:
        return app[username]
    else:
        return None


def retrieve_gallery(user):
    if 'gallery' in user:
        return user['gallery']
    return None


def retrieve_about(user):
    if 'about' in user:
        return user['about']
    return None


def bootstrap_db():
    return PersistentContainer('', None)


def appmaker(zodb_root):
    if not 'pyGallerid-app-root' in zodb_root:
        app_root = bootstrap_db()
        zodb_root['pyGallerid-app-root'] = app_root
        import transaction
        transaction.commit()
    return zodb_root['pyGallerid-app-root']



from pyramid.view import view_config
from pyramid.httpexceptions import (
    HTTPFound,
    HTTPNotFound,
    HTTPForbidden,
)

from ..models import retrieve_gallery
from ..models.user import User


@view_config(context=User)
def user(context, request):
    gallery = retrieve_gallery(context)
    if gallery is not None:
        return HTTPFound(location=request.resource_url(gallery))
    else:
        return HTTPNotFound()



# -*- coding: utf-8 -*-

"""
Provides models for the gallery resources of pyGallerid.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp


import os
import shutil
import datetime

from persistent import Persistent

from . import PersistentLocationAware, PersistentOrderedContainer


class GalleryDocument(Persistent, PersistentLocationAware):
    __attributes__ = ('description', 'long_description')

    def __init__(self, name, description, long_description, parent=None):
        Persistent.__init__(self)
        PersistentLocationAware.__init__(self, name, parent)
        self.description = description
        self.long_description = long_description


class GalleryContainer(PersistentOrderedContainer):
    __attributes__ = ('path', 'absolute_path', 'description',
                       'preview_picture', 'preview_size')

    def __init__(self, name, description=None, preview_picture=None,
                 path=None, parent=None):
        PersistentOrderedContainer.__init__(self, name, parent)
        if description is None:
            self.description = name
        else:
            self.description = description
        self.__preview_picture = None
        self.preview_size = (-1, -1)
        self.__path = path

    @property
    def path(self):
        return self.__path

    @path.setter
    def path(self, new_path):
        if self.__path is not None:
            try:
                abs_path_tuple = self.absolute_path_tuple
                abs_parent_path = os.path.join(*abs_path_tuple[:-1])
                old_abs_path = os.path.join(abs_parent_path, self.path)
                new_abs_path = os.path.join(abs_parent_path, new_path)
                if os.path.exists(new_abs_path):
                    if os.path.isdir(new_abs_path):
                        for file in os.listdir(old_abs_path):
                            shutil.move(os.path.join(old_abs_path, file),
                                        new_abs_path)
                        os.rmdir(old_abs_path)
                    else:
                        raise ValueError('The new path already exists'
                                         ' and is not a directory')
                else:
                    shutil.move(old_abs_path, new_abs_path)
            except OSError, shutil.Error:
                print 'Unable to change path of gallery container:', self.name
                raise
        self.__path = new_path

    @property
    def absolute_path(self):
        if self.path is None:
            raise AttributeError('The container has no valid path')
        if isinstance(self.parent, GalleryContainer):
            return os.path.join(self.parent.absolute_path, self.path)
        else:
            return self.path

    @property
    def absolute_path_tuple(self):
        if self.path is None:
            raise AttributeError('The container has no valid path')
        if isinstance(self.parent, GalleryContainer):
            return self.parent.absolute_path_tuple + (self.path,)
        else:
            return (self.path,)

    @property
    def preview_picture(self):
        if self.__preview_picture is None and len(self) > 0:
            child = self.get_children(0)
            if isinstance(child, GalleryPicture):
                return child
            else:
                return child.preview_picture
        else:
            return self.__preview_picture

    @preview_picture.setter
    def preview_picture(self, picture):
        assert isinstance(picture, GalleryPicture) \
               or isinstance(picture, GalleryContainer)
        if isinstance(picture, GalleryContainer):
            picture = picture.preview_picture
        if self.has_picture(picture):
            self.__preview_picture = picture
        else:
            raise ValueError("The container must contain it's "
                             "own preview picture")

    def has_picture(self, picture):
        assert isinstance(picture, GalleryPicture)
        found = False
        for child in self.itervalues():
            if isinstance(child, GalleryContainer):
                if child.has_picture(picture):
                    found = True
                    break
            elif isinstance(child, GalleryPicture):
                if picture == child:
                    found = True
                    break
        return found


class Gallery(GalleryContainer):
    __attributes__ = ['user']
    __attributes__.extend(GalleryContainer.__attributes__)

    def __init__(self, description=None, path=None,
                 user=None, name='gallery', parent=None):
        GalleryContainer.__init__(self, name, description,
                                  path=path, parent=parent)
        self.user = user


class GalleryAlbum(GalleryContainer):
    __attributes__ = ['long_description', 'location', 'date_from', 'date_to']
    __attributes__.extend(GalleryContainer.__attributes__)

    def __init__(self, name, description=None, \
                 long_description=None, location=None, \
                 date_from=datetime.datetime.now(), date_to=None,
                 path=None, parent=None):
        GalleryContainer.__init__(self, name, description,
                                  path=path, parent=parent)
        self.long_description = long_description
        self.location = location
        self.date_from = date_from
        self.date_to = date_to

    pictures = property(lambda self: self.children)

    @pictures.setter
    def pictures(self, pictures):
        self.children = pictures


class GalleryPicture(Persistent, PersistentLocationAware):
    __attributes__ = ['big_image_view', 'regular_image_view',
                      'small_image_view', 'description', 'location',
                      'date']

    def __init__(self, name, big_image_view, regular_image_view,
                 small_image_view, description=None, location=None,
                 date=datetime.datetime.now(), parent=None):
        Persistent.__init__(self)
        PersistentLocationAware.__init__(self, name, parent)
        if isinstance(big_image_view, GalleryImageFile):
            self.big_image_view = GalleryImageView(big_image_view)
        else:
            assert isinstance(big_image_view, GalleryImageView)
            self.big_image_view = big_image_view
        if isinstance(regular_image_view, GalleryImageFile):
            self.regular_image_view = GalleryImageView(regular_image_view)
        else:
            assert isinstance(regular_image_view, GalleryImageView)
            self.regular_image_view = regular_image_view
        if isinstance(small_image_view, GalleryImageFile):
            self.small_image_view = GalleryImageView(small_image_view)
        else:
            assert isinstance(small_image_view, GalleryImageView)
            self.small_image_view = small_image_view
        self.description = description
        self.location = location
        self.date = date

    @property
    def small_image_path(self):
        assert self.parent is not None
        assert isinstance(self.parent, GalleryAlbum)
        return os.path.join(self.parent.absolute_path,
                            self.small_image_view.image.filename)

    @property
    def regular_image_path(self):
        assert self.parent is not None
        assert isinstance(self.parent, GalleryAlbum)
        return os.path.join(self.parent.absolute_path,
                            self.regular_image_view.image.filename)

    @property
    def big_image_path(self):
        assert self.parent is not None
        assert isinstance(self.parent, GalleryAlbum)
        return os.path.join(self.parent.absolute_path,
                            self.big_image_view.image.filename)


class GalleryImageView(Persistent):
    def __init__(self, image, view_size=None, crop_rect=None):
        self.image = image
        self.view_size = view_size
        self.crop_rect = crop_rect

    def __str__(self):
        return self.image.filename

    @property
    def width(self):
        if self.view_size is None:
            return self.image.width
        return self.view_size[0]

    @property
    def height(self):
        if self.view_size is None:
            return self.image.height
        return self.view_size[1]

    @property
    def crop_x(self):
        if self.crop_rect is None:
            return 0
        return self.crop_rect[0]

    @property
    def crop_y(self):
        if self.crop_rect is None:
            return 0
        return self.crop_rect[1]

    @property
    def crop_width(self):
        if self.crop_rect is None:
            return self.width
        return self.crop_rect[2] - self.crop_rect[0]

    @property
    def crop_height(self):
        if self.crop_rect is None:
            return self.height
        return self.crop_rect[3] - self.crop_rect[1]


class GalleryImageFile(Persistent):
    def __init__(self, filename, size=(-1, -1), tags=None):
        self.filename = filename
        self.size = size
        self.tags = tags

    @property
    def width(self):
        return self.size[0]

    @property
    def height(self):
        return self.size[1]

    def __contains__(self, tag):
        return tag in self.tags

    def __getitem__(self, tag):
        return self.tags[tag]



# -*- coding: utf-8 -*-

"""
Setuptools setup.py file for pyGallerid.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp


import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = [
    'pyramid',
    'pyramid_debugtoolbar',
    'pyramid_beaker',
    'pyramid_tm',
    'ZODB3',
    'pyramid_zodbconn',
    #'repoze.zodbconn',
    'transaction',
    'PIL',
    'python-dateutil==1.5',
    'repoze.evolution',
]

setup(name='pyGallerid',
    version='0.2',
    description='python web photo gallery based on pyramid',
    long_description=README + '\n\n' +  CHANGES,
    classifiers=[
        "Operating System :: OS Independent",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Development Status :: 4 - Beta",
        "Framework :: Pylons",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
      ],
    author='Benjamin Hepp',
    author_email='benjamin.hepp@gmail.com',
    license='BSD',
    url='https://github.com/bennihepp/pyGallerid',
    keywords='web pyramid pylons gallery photo',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=requires,
    tests_require=requires,
    test_suite="pyGallerid",
    entry_points = """\
    [paste.app_factory]
    main = pyGallerid:main
    [console_scripts]
    init_gallery = pyGallerid.scripts.init_gallery:main
    import_pictures = pyGallerid.scripts.import_pictures:main
    """,
)




# -*- coding: utf-8 -*-

"""
Creates the pyGallerid WSGI application instance.
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp

from pyramid.config import Configurator
from pyramid_zodbconn import get_connection, db_from_uri
from pyramid.settings import asbool
from pyramid.authentication import SessionAuthenticationPolicy
from pyramid.authorization import ACLAuthorizationPolicy

from pyramid_beaker import session_factory_from_settings, \
                           set_cache_regions_from_settings

from models import appmaker, __sw_version__

# This is only needed when using SQLAlchemy
#from sqlalchemy import engine_from_config
#from .models import DBSession

__version__ = 0.2


def root_factory(request):
    conn = get_connection(request)
    return appmaker(conn.root())


def evolve_database(root, sw_version, initial_db_version=0):
    from repoze.evolution import ZODBEvolutionManager, evolve_to_latest
    manager = ZODBEvolutionManager(
        root, evolve_packagename='pyGallerid.evolve',
        sw_version=sw_version,
        initial_db_version=initial_db_version
    )
    evolve_to_latest(manager)


def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
    if asbool(settings.get('wingdbstub', 'false')):
        import misc.wingdbstub

    # evolve ZODB database to latest version
    db = db_from_uri(settings['zodbconn.uri'])
    conn = db.open()
    evolve_database(appmaker(conn.root()), __sw_version__)
    db.close()

    # set up beaker session
    session_factory = session_factory_from_settings(settings)
    # set up beaker cache
    set_cache_regions_from_settings(settings)

    config = Configurator(root_factory=root_factory, settings=settings)

    config.include(set_auth_policies)

    config.include(add_user_property)

    # include pyramid plugins
    config.include('pyramid_beaker')
    config.include('pyramid_tm')
    config.include('pyramid_zodbconn')
    config.include('pyramid_rewrite')

    # register beaker session factory
    config.set_session_factory(session_factory)

    # This is only needed when using SQLAlchemy
    #engine = engine_from_config(settings, 'sqlalchemy.')
    #DBSession.configure(bind=engine)

    # add static views
    config.include(add_static_views)

    # scan for declarative configurations
    config.scan()

    # add url rewriting rules
    config.include(add_rewrite_rules)

    # return WSGI application instance
    return config.make_wsgi_app()


def add_user_property(config):
    from pyramid.traversal import find_root
    from pyramid.security import authenticated_userid
    from models import retrieve_user

    # add user property to request objects
    def get_user(request):
        root = find_root(request.context)
        username = authenticated_userid(request)
        user = retrieve_user(root, username)
        return user
    config.set_request_property(get_user, 'user', reify=True)


def set_auth_policies(config):
    from models.user import groupfinder
    # set up authentication and authorization
    authn_policy = SessionAuthenticationPolicy(callback=groupfinder)
    authz_policy = ACLAuthorizationPolicy()
    config.set_authentication_policy(authn_policy)
    config.set_authorization_policy(authz_policy)


def add_rewrite_rules(config):
    pattern = r'/'
    target = '/%s/gallery/' % config.registry.settings['default_user']
    config.add_rewrite_rule(pattern, target)
    pattern = r'/favicon.ico'
    target = '/static/favicon.ico'
    config.add_rewrite_rule(pattern, target)
    pattern = r'/gallery/(?P<subpath>.*)'
    target = '/%s/gallery/%%(subpath)s' % config.registry.settings['default_user']
    config.add_rewrite_rule(pattern, target)


def add_static_views(config):
    settings = config.registry.settings
    config.add_static_view(
        'static',
        settings['static_dir'],
        cache_max_age=3600
    )
    config.add_static_view(
        'pictures',
        settings['image_dir'],
        cache_max_age=3600
    )
